﻿#region GPL License

//Mediaportal (http://www.team-mediaportal.com/) Plugin - WatchedSynchronizer
//Copyright (C) 2015 Max Wimmelbacher
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#endregion

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MediaPortal.GUI.Library;

namespace WatchedSynchronizer
{

  /// <summary>
  /// The class WatchedSynchronizer.AdditionalMovingPicturesDatabase provides all necessary properties/methods to interact with a moving pictures database.
  /// </summary>
  
  class AdditionalMovingPicturesDatabase
  {
    #region Declaration

    private string mDatabaseFile;
    private AdditionalMovingPicturesDatabaseSQLite mDatabase;

    #endregion Declaration;

    #region Constructors

    public AdditionalMovingPicturesDatabase(string strDatabaseFile)
    {
      mDatabaseFile = strDatabaseFile;
      mDatabase = AdditionalDatabaseFactory.GetMovingPicturesDatabase(mDatabaseFile);
    }

    #endregion

    #region Properties

    public string DatabaseName
    {
      get
      {
        if (mDatabase != null)
        {
          return mDatabase.DatabaseName;
        }
        return string.Empty;
      }
    }

    #endregion

    #region Public methods

    public void ReOpen()
    {
      Dispose();
      mDatabase = AdditionalDatabaseFactory.GetMovingPicturesDatabase(mDatabaseFile);
    }

    public void Dispose()
    {
      if (mDatabase != null)
      {
        mDatabase.Dispose();
        Log.Info("WatchedSynchronizer: MovingPictures database closed.");
      }
      mDatabase = null;
    }

    //Gets the id from local_media table (unique key to identify an movie) based on the path and filename.
    public string GetUserMovieSettingsId(string strFilenameAndPath, ref string strMovieInfoID)
    {
        string strLocalMediaID = GetLocalMediaID(strFilenameAndPath);
        if (strLocalMediaID == string.Empty)
        {
            Log.Debug("WatchedSynchronizer: File '" + strFilenameAndPath + "' is not available in table local_media in database '" + mDatabase.DatabaseName + "'.");
            return string.Empty;
        }
        //Get the user UserMovieSettings Id
        return mDatabase.GetUserMovieSettingsId(strLocalMediaID, ref strMovieInfoID);
    }

    //Gets the id from local_media table (unique key to identify an movie) based on the path and filename.
    public string GetLocalMediaID(string strFilenameAndPath)
    {
        return mDatabase.GetLocalMediaID(strFilenameAndPath);
    }

    //Gets the given int setting from table settings
    public int GetIntSetting(string strSettingName, ref int iOutValue)
    {
        return mDatabase.GetIntSetting(strSettingName, ref iOutValue);
    }

    //GetMovieWatchedStatus, get the current watch state
    public void GetMovieWatchedStatus(string strUserMovieSettingsId, ref bool bolWatched, ref int intResumePart, ref int intResumeTime, ref byte[] bteResumeData, ref int intResumeTitleBD)
    {
        mDatabase.GetMovieWatchedStatus(strUserMovieSettingsId, ref bolWatched, ref intResumePart, ref intResumeTime, ref bteResumeData, ref intResumeTitleBD);
    }

    //SetMovieWatchedStatus, set the current watch state
    public void SetMovieWatchedStatus(string strUserMovieSettingsId, bool bolWatched, int intResumePart, int intResumeTime, byte[] bteResumeData, int intResumeTitleBD)
    {
        mDatabase.SetMovieWatchedStatus(strUserMovieSettingsId, bolWatched, intResumePart, intResumeTime, bteResumeData, intResumeTitleBD);
    }

    //AddToWatchedHistory, add an watch history entry with datetime = now
    public void AddToWatchedHistory(string strMovieInfoID)
    {
        mDatabase.AddToWatchedHistory(strMovieInfoID);
    }

    #endregion

  }
}
