﻿#region GPL License

//Mediaportal (http://www.team-mediaportal.com/) Plugin - WatchedSynchronizer
//Copyright (C) 2015 Max Wimmelbacher
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#endregion

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MediaPortal.GUI.Library;
using MediaPortal.Database;
using SQLite.NET;
using WindowPlugins.GUITVSeries;
using System.Globalization;
using Cornerstone.Database.CustomTypes;
using System.Reflection;

namespace WatchedSynchronizer
{

  /// <summary>
    /// The class WatchedSynchronizer.AdditionalMovingPicturesDatabaseSQLite provides all necessary properties/methods which do the real communication with a moving pictures database.
  /// Objects created from this class are returned by calls to methods of the class AdditionalDatabaseFactory,
  /// </summary>

  class AdditionalMovingPicturesDatabaseSQLite : IDisposable
  {

    #region Declaration

    private Dictionary<string, SQLiteResultCacheItem> mCache;
    private SQLiteClient mDatabase;
    private string mDatabaseFile;
    private PropertyInfo propertyInfo;

    #endregion

    #region Constructors

    public AdditionalMovingPicturesDatabaseSQLite(string strDatabaseFile)
    {
      mDatabaseFile = strDatabaseFile;
      Open();
    }

    #endregion

    #region Properties

    public string DatabaseName
    {
      get
      {
        return mDatabaseFile;
      }
    }

    #endregion Properties

    #region Public methods

    public void Dispose()
    {
      mDatabase.Close();
      mDatabase = null;
    }

    public string GetLocalMediaID(string strFilenameAndPath)
    {
      try
      {
        string strLocalMediaID = string.Empty;
        strFilenameAndPath = strFilenameAndPath.Trim();
        DatabaseUtility.RemoveInvalidChars(ref strFilenameAndPath);
        string strSQL = String.Format("SELECT * FROM local_media WHERE fullpath = '{0}'", strFilenameAndPath);
        Log.Debug("WatchedSynchronizer: (GetCompositeId) SQL statement '" + strSQL + "' is going to be executed in database '" + mDatabase.DatabaseName + "'.");
        SQLiteResultSet objResults = mDatabase.Execute(strSQL);
        if (objResults.Rows.Count > 0)
        {
            strLocalMediaID = DatabaseUtility.Get(objResults, 0, "id");
            AddToCache("local_media", strLocalMediaID, objResults);
        }
        return strLocalMediaID;
      }
      catch (Exception ex)
      {
        Log.Error("WatchedSynchronizer: MovingPictures database exception err:{0} stack:{1}", ex.Message, ex.StackTrace);
      }
      return string.Empty;
    }

    public int GetIntSetting(string strSettingName, ref int iOutValue)
    {
      try
      {
        //Error return Value is -1 = exception or other error
        iOutValue = -1;
        strSettingName = strSettingName.Trim();
        DatabaseUtility.RemoveInvalidChars(ref strSettingName);
        string strSQL = String.Format("SELECT * FROM settings WHERE key = '{0}'", strSettingName);
        Log.Debug("WatchedSynchronizer: (GetCompositeId) SQL statement '" + strSQL + "' is going to be executed in database '" + mDatabase.DatabaseName + "'.");
        SQLiteResultSet objResults = mDatabase.Execute(strSQL);
        if (objResults.Rows.Count > 0)
        {
            iOutValue = DatabaseUtility.GetAsInt(objResults, 0, "value");
            AddToCache(DBEpisode.cTableName, strSettingName, objResults);
            //return Value 1 = OK
            return 1;
        }
        //Error return Value 0 = not found
        return 0;
      }
      catch (Exception ex)
      {
        Log.Error("WatchedSynchronizer: MovingPictures database exception err:{0} stack:{1}", ex.Message, ex.StackTrace);
      }
      //Error return Value is -1 = exception or other error
      iOutValue = -1;
      return -1;
    }

    public string GetUserMovieSettingsId(string strLocalMediaID, ref string strMovieInfoID)
    {
        try
        {
            string strUserMovieSettingsId = string.Empty;
            if (CheckCache("local_media__movie_info", strLocalMediaID, "local_media_id", ref strMovieInfoID) == false)
            {
                string strSQL = String.Format("SELECT * FROM local_media__movie_info WHERE local_media_id = '{0}'", strLocalMediaID);
                Log.Debug("WatchedSynchronizer: (MovieInfoID) SQL statement '" + strSQL + "' is going to be executed in database '" + mDatabase.DatabaseName + "'.");
                SQLiteResultSet objResults = mDatabase.Execute(strSQL);
                if (objResults.Rows.Count > 0)
                {
                    strMovieInfoID = DatabaseUtility.Get(objResults, 0, "movie_info_id");
                    AddToCache("local_media__movie_info", strLocalMediaID, objResults);
                }
            }
            if (strMovieInfoID != string.Empty)
            {
                if (CheckCache("movie_info__user_movie_settings", strMovieInfoID, "movie_info_id", ref strUserMovieSettingsId) == false)
                {
                    string strSQL = String.Format("SELECT * FROM movie_info__user_movie_settings WHERE movie_info_id = '{0}'", strMovieInfoID);
                    Log.Debug("WatchedSynchronizer: (GetUserMovieSettingsId) SQL statement '" + strSQL + "' is going to be executed in database '" + mDatabase.DatabaseName + "'.");
                    SQLiteResultSet objResults = mDatabase.Execute(strSQL);
                    if (objResults.Rows.Count > 0)
                    {
                        strUserMovieSettingsId = DatabaseUtility.Get(objResults, 0, "user_movie_settings_id");
                        AddToCache("movie_info__user_movie_settings", strMovieInfoID, objResults);
                    }
                }
            }
            else
            {
                Log.Debug("WatchedSynchronizer: strMovieInfoID is empty! Could not get UserMovieSettingsId");
            }
            return strUserMovieSettingsId;
        }
        catch (Exception ex)
        {
            Log.Error("WatchedSynchronizer: GetUserMovieSettingsId: MovingPictures database exception err:{0} stack:{1}", ex.Message, ex.StackTrace);
        }
        return string.Empty;
    }

    public bool GetMovieWatchedStatus(string strUserMovieSettingsId, ref bool bolWatched, ref int intResumePart, ref int intResumeTime, ref byte[] bteResumeData, ref int intResumeTitleBD)
    {
        try
        {
            //string strEpisodeWatchedStatus = string.Empty;
            //if (CheckCache("user_movie_settings", strUserMovieSettingsId, DBOnlineEpisode.cWatched, ref strEpisodeWatchedStatus) == false)
            //{
            string strSQL = String.Format("SELECT * FROM user_movie_settings WHERE id = '{0}'", strUserMovieSettingsId);
                Log.Debug("WatchedSynchronizer: (GetEpisodeWatchedStatus) SQL statement '" + strSQL + "' is going to be executed in database '" + mDatabase.DatabaseName + "'.");
                SQLiteResultSet objResults = mDatabase.Execute(strSQL);
                if (objResults.Rows.Count > 0)
                {
                    bolWatched = Convert.ToBoolean(DatabaseUtility.GetAsInt(objResults, 0, "watched"));
                    intResumePart = DatabaseUtility.GetAsInt(objResults, 0, "resume_part");
                    intResumeTime = DatabaseUtility.GetAsInt(objResults, 0, "resume_time");

                    //Get Resumedata and convert it to byte array
                    //TODO: throws exception!?!?!?!
                    //string resumeString = DatabaseUtility.Get(objResults, 0, "resume_data");
                    //ByteArray newObj = new ByteArray();
                    //newObj.LoadFromString(resumeString);
                    //Log.Debug("WatchedSynchronizer: len = " + newObj.ToString());
                    //bteResumeData = newObj.Data;


                    intResumeTitleBD = DatabaseUtility.GetAsInt(objResults, 0, "resume_titlebd");
                    //AddToCache(DBOnlineEpisode.cTableName, strCompositeId, objResults);
                    return true;
                }
            //}
                return false;
        }
        catch (Exception ex)
        {
            Log.Error("WatchedSynchronizer: GetMovieWatchedStatus: MovingPictures database exception err:{0} stack:{1}", ex.Message, ex.StackTrace);
        }
        return true;
    }

    //SetMovieWatchedStatus, set the current watch state
    public void SetMovieWatchedStatus(string strUserMovieSettingsId, bool bolWatched, int intResumePart, int intResumeTime, byte[] bteResumeData, int intResumeTitleBD)
    {
        try
        {
            int intWatchedStatus;
            if (bolWatched == true)
            {
                intWatchedStatus = 1;
            }
            else
            {
                intWatchedStatus = 0;
            }
            string strWatchedStatus = intWatchedStatus.ToString();

            string strSQL = String.Format("UPDATE user_movie_settings SET watched = {0}, resume_part = {1}, resume_time = {2}, resume_data = NULL, resume_titlebd = {3} WHERE id = '{4}'", strWatchedStatus, intResumePart.ToString(), intResumeTime.ToString(), intResumeTitleBD.ToString(), strUserMovieSettingsId);
            Log.Debug("WatchedSynchronizer: (SetEpisodeWatchedStatus) SQL statement '" + strSQL + "' is going to be executed in database '" + mDatabase.DatabaseName + "'.");
            mDatabase.Execute(strSQL);
        }
        catch (Exception ex)
        {
            Log.Error("WatchedSynchronizer: SetMovieWatchedStatus: MovingPictures database exception err:{0} stack:{1}", ex.Message, ex.StackTrace);
        }
    }

    //AddToWatchedHistory, add an watch history entry with datetime = UtcNow
    public void AddToWatchedHistory(string strMovieInfoID)
    {
        try
        {
            string strSQL = String.Format("INSERT INTO watched_history VALUES (NULL, 1, {0}, '{1}Z')", strMovieInfoID, DateTime.UtcNow.ToString("yyyy-MM-dd HH:mm:ss"));
            Log.Debug("WatchedSynchronizer: (SetEpisodeWatchedStatus) SQL statement '" + strSQL + "' is going to be executed in database '" + mDatabase.DatabaseName + "'.");
            mDatabase.Execute(strSQL);
        }
        catch (Exception ex)
        {
            Log.Error("WatchedSynchronizer: AddToWatchedHistory: MovingPictures database exception err:{0} stack:{1}", ex.Message, ex.StackTrace);
        }
    }

    #endregion

    #region Private methods

    private void Open()
    {
      mCache = new Dictionary<string, SQLiteResultCacheItem>();
      Log.Info("WatchedSynchronizer: Opening MovingPictures database " + mDatabaseFile);
      try
      {
        if (mDatabase != null)
        {
          Log.Info("WatchedSynchronizer: MovingPictures database already opened.");
          return;
        }
        mDatabase = new SQLiteClient(mDatabaseFile);
        DatabaseUtility.SetPragmas(mDatabase);
        Log.Info("WatchedSynchronizer: MovingPictures database opened.");
      }
      catch (Exception ex)
      {
        Log.Error("WatchedSynchronizer: MovingPictures database exception err:{0} stack:{1}", ex.Message, ex.StackTrace);
      }
    }

    private bool CheckCache(string strTableName, string strKey, string strField, ref string strValue)
    {
        if (mCache.ContainsKey(strTableName) == true)
        {
            if (mCache[strTableName].Key == strKey && mCache[strTableName].IsDirty == false)
            {
                if (strValue == string.Empty)
                {
                    strValue = (string)DatabaseUtility.Get(mCache[strTableName].ResultSet, 0, strField);
                    return true;
                }
                else if (strValue == (string)DatabaseUtility.Get(mCache[strTableName].ResultSet, 0, strField))
                {
                    return true;
                }
                else
                {
                 mCache[strTableName].IsDirty = true;
                }
            }
        }
        return false;
    }

    private void AddToCache(string strTableName, string strKey, SQLiteResultSet objResultSet)
    {
        if (mCache.ContainsKey(strTableName) == true)
        {
            mCache.Remove(strTableName);
        }
        SQLiteResultCacheItem objCacheItem = new SQLiteResultCacheItem(strKey, objResultSet);
        mCache.Add(strTableName, objCacheItem);
    }

    #endregion

  }
}